@@ -1,4 +1,4 @@
-;;; diff-ansi.el --- Display diffs using alternative diffing tools -*- lexical-binding: t -*-
+;;; diff-ansi.el --- Display diff's using alternative diffing tools -*- lexical-binding: t -*-
 
 ;; SPDX-License-Identifier: GPL-2.0-or-later
 ;; Copyright (C) 2021  Campbell Barton
@@ -6,14 +6,15 @@
 ;; Author: Campbell Barton <ideasman42@gmail.com>
 
 ;; URL: https://codeberg.org/ideasman42/emacs-diff-ansi
-;; Version: 0.2
+;; Package-Version: 20250913.2251
+;; Package-Revision: 1ae63be2381d
 ;; Package-Requires: ((emacs "29.1"))
 
 ;;; Commentary:
 
 ;; Support showing color diffs using external tools.
 
-;;; Usage:
+;;; Usage
 
 ;; See readme.rst.
 ;;
@@ -43,13 +44,14 @@
 ;; Custom Variables
 
 (defgroup diff-ansi nil
-  "Display diffs using external display commands."
-  :group 'diff)
+  "Configure smooth scrolling when jumping to new locations."
+  :group 'scrolling)
 
 (defcustom diff-ansi-use-magit-revision-diff t
   "Use diff-ansi for `magit-revision-mode'."
   :type 'boolean)
 
+
 (defcustom diff-ansi-tool 'delta
   "Command to use for generating the diff."
   :type
@@ -59,10 +61,9 @@
 
            (symbol :tag "Use `diff-ansi-tool-custom' command." custom)))
 
-;; Note that this has its values extracted and isn't used directly.
-(defface diff-ansi-default-face (list (list t :foreground "white" :background "black"))
-  "Face used to render black color."
-  :group 'diff-ansi)
+;; Note that this has it's values extracted and isn't used directly.
+(defface diff-ansi-default-face (list (list t :foreground "black" :background "black"))
+  "Face used to render black color.")
 
 (defcustom diff-ansi-tool-custom nil
   "Command to use when `diff-ansi-tool' is set to `custom'.
@@ -97,13 +98,13 @@ This must take the diff content as the `standard-input'."
 (defcustom diff-ansi-chunks-size 16384
   "Number of characters to process at once.
 Used for `progressive' and `multiprocess' methods."
-  :type 'natnum)
+  :type 'integer)
 
 (defcustom diff-ansi-multiprocess-jobs nil
   "Number of simultaneous jobs to launch when using `multiprocess' method.
 
-A nil value auto-detects the number of processors using `num-processors'."
-  :type '(choice (const :tag "Auto-detect" nil) (natnum :tag "Number of jobs")))
+A nil value detects the number of processes on the system (when supported)."
+  :type 'integer)
 
 (defcustom diff-ansi-verbose-progress nil
   "When enabled, show the progress of progressive conversion in the echo area.
@@ -154,8 +155,8 @@ It can be useful to show progress when viewing very large diffs."
 ;; ---------------------------------------------------------------------------
 ;; Forward Declarations
 
-;; Needed since the `eval' causes the function to be hidden.
-(declare-function diff-ansi--ansi-color-apply-on-region-with-bg-impl "ansi-color")
+;; Needed since the the `eval' causes the function to be hidden.
+(declare-function diff-ansi--ansi-color-apply-on-region-with-bg-impl "diff-ansi")
 (declare-function magit-wash-sequence "magit-section")
 
 
@@ -165,7 +166,7 @@ It can be useful to show progress when viewing very large diffs."
 (defmacro diff-ansi--with-advice (advice &rest body)
   "Execute BODY with ADVICE temporarily enabled.
 
-Each advice is a triplet of (SYMBOL HOW FUNCTION),
+Advice are triplets of (SYMBOL HOW FUNCTION),
 see `advice-add' documentation."
   (declare (indent 1))
   (let ((advice-list advice)
@@ -214,9 +215,9 @@ see `advice-add' documentation."
          (message omessage)))))
 
 (defmacro diff-ansi--with-temp-directory (name &rest body)
-  "Bind NAME to the name of a new temporary directory and evaluate BODY.
+  "Bind NAME to the name of a new temporary file and evaluate BODY.
 Delete the temporary file after BODY exits normally or non-locally.
-NAME will be bound to the file name of the temporary directory.
+NAME will be bound to the file name of the temporary file.
 
 The following keyword arguments are supported:
 
@@ -225,7 +226,7 @@ The following keyword arguments are supported:
 
 :suffix STRING
   If non-nil, pass STRING to `make-temp-file' as the SUFFIX argument."
-  (declare (indent 1) (debug (sexp body)))
+  (declare (indent 1) (debug (symbolp body)))
   (unless (symbolp name)
     (error "Expected name to be as symbol, found %S" (type-of name)))
   (let ((keyw nil)
@@ -242,11 +243,11 @@ The following keyword arguments are supported:
          (pop body))))
     (when extra-keywords
       (error "Invalid keywords: %s" (mapconcat #'symbol-name extra-keywords " ")))
-    (let ((temp-file (make-symbol "temp-file")))
-      (setq prefix (or prefix ""))
-      (setq suffix (or suffix ""))
+    (let ((temp-file (make-symbol "temp-file"))
+          (prefix (or prefix ""))
+          (suffix (or suffix "")))
       `(let* ((,temp-file (file-name-as-directory (make-temp-file ,prefix t ,suffix nil)))
-              (,name ,temp-file))
+              (,name (file-name-as-directory ,temp-file)))
          (unwind-protect
              (progn
                ,@body)
@@ -293,7 +294,7 @@ Additional keyword arguments may also be passed in.
                 (:input
                  (cond
                   ((null v)) ; No input.
-                  ((eq v t)) ; True for string input (standard input data).
+                  ((eq v t)) ; String input (standard input data).
                   ((stringp v))
                   ((bufferp v)
                    (unless (buffer-live-p v)
@@ -303,14 +304,14 @@ Additional keyword arguments may also be passed in.
                  (setq input v))
                 (:output
                  (cond
-                  ((null v)) ; No output.
-                  ((eq v t)) ; String output (file path).
-                  ((stringp v)
-                   (when (string-equal v "")
+                  ((null output)) ; No output.
+                  ((eq output t)) ; String output (file path).
+                  ((stringp output)
+                   (when (string-equal output "")
                      (error "Empty string used as file-path")))
-                  ((bufferp v)
-                   (unless (buffer-live-p v)
-                     (error "Output buffer is invalid %S" v)))
+                  ((bufferp output)
+                   (unless (buffer-live-p output)
+                     (error "Input buffer is invalid %S" output)))
 
                   (t
                    (error "Output expected a buffer, string or nil")))
@@ -371,7 +372,7 @@ Additional keyword arguments may also be passed in.
                   (erase-buffer))))
 
               (process-send-eof proc)
-              (while (null sentinel-called)
+              (while (not sentinel-called)
                 (accept-process-output))
 
               ;; Check exit-code.
@@ -446,21 +447,22 @@ Optional keywords in KEYWORDS.
                 ((null v)) ; Ignore.
                 ((floatp v))
                 (t
-                 (error "Idle expected a float, not a %S" (type-of v))))
+                 (error "Argument :idle expected a float, not a %S" (type-of v))))
                (setq idle v))
 
               (:jobs
                (cond
-                ((null v)) ; Ignore (detect).
-                ((integerp v)) ; Integer jobs.
+                ((null v)) ; Ignore.
+                ((eq v t)) ; String output (result vector)
+                ((symbolp v))
                 (t
-                 (error "Jobs expected nil or an integer")))
+                 (error "Output expected nil, t or a symbol")))
                (setq jobs v))
 
               (:output
                (cond
                 ((null v)) ; Ignore.
-                ((eq v t)) ; True for string output.
+                ((eq v t)) ; String output (result vector)
                 ((functionp v)
                  (setq output-is-fn t))
                 (t
@@ -470,7 +472,8 @@ Optional keywords in KEYWORDS.
               (:progress-message
                (cond
                 ((null v)) ; Ignore.
-                ((stringp v)) ; String message.
+                ((stringp v)
+                 (setq output-is-fn t))
                 (t
                  (error "Progress-message expected nil or string, not a %S" (type-of v))))
                (setq progress-message v))
@@ -485,13 +488,14 @@ Optional keywords in KEYWORDS.
     (unless idle
       (setq idle 0.01))
 
+    ;; TODO: detect.
     (unless jobs
       (setq jobs
             (cond
-             (diff-ansi-multiprocess-jobs
-              diff-ansi-multiprocess-jobs)
+             ((fboundp 'num-processors) ; Emacs 29+
+              (* 2 (num-processors)))
              (t
-              (* 2 (num-processors))))))
+              (or diff-ansi-multiprocess-jobs 64)))))
 
     (let* ((args-len (length args))
            (results (make-vector args-len nil))
@@ -551,7 +555,7 @@ Optional keywords in KEYWORDS.
 
       ;; Setup indices to represent jobs that aren't complete.
       (let ((i args-len))
-        (while (null (zerop i))
+        (while (not (zerop i))
           (push (decf i) jobs-pending)))
 
       ;; Main loop to complete
@@ -597,10 +601,12 @@ Optional keywords in KEYWORDS.
   "Return the HEX color from FACE and ATTR."
   (declare (important-return-value t))
   (apply #'format
-         "#%02x%02x%02x"
-         ;; Shift by -8 to map the value returned by `color values':
-         ;; 0..65535 to 0..255 for `#RRGGBB` string formatting.
-         (mapcar (lambda (n) (ash n -8)) (color-values (face-attribute face attr)))))
+         (cons
+          "#%02x%02x%02x"
+          (mapcar
+           ;; Shift by -8 to map the value returned by `color values':
+           ;; 0..65535 to 0..255 for `#RRGGBB` string formatting.
+           (lambda (n) (ash n -8)) (color-values (face-attribute face attr))))))
 
 
 ;; ---------------------------------------------------------------------------
@@ -687,7 +693,7 @@ Store the result in TARGET-BUF when non-nil."
 Argument BEG is only used to calculate the progress percentage."
   (declare (important-return-value nil))
   (cond
-   ((null (buffer-live-p buf))
+   ((not (buffer-live-p buf))
     ;; The buffer was closed (most likely by the user) while the timer was running,  cancel it.
     (cancel-timer timer))
    ((input-pending-p)
@@ -763,12 +769,13 @@ Store the result in TARGET-BUF when non-nil."
         (diff-ansi--call-process-pipe-chain diff-command :input diff-str :output (current-buffer))
         (setq end (point))
 
-        (letrec ((timer
-                  (run-at-time 0.0 0.001
-                               (lambda (buf beg range)
-                                 (diff-ansi-progressive-highlight-impl buf beg range timer))
-                               (current-buffer) beg (cons beg end))))
-          (setq diff-ansi--ansi-color-timer timer))))))
+        ;; Postpone activation until the timer can take it's self as an argument.
+        (diff-ansi--with-advice ((#'timer-activate :override (lambda (&rest _) nil)))
+          (setq diff-ansi--ansi-color-timer (run-at-time 0.0 0.001 nil))
+          (timer-set-function diff-ansi--ansi-color-timer #'diff-ansi-progressive-highlight-impl
+                              (list
+                               (current-buffer) beg (cons beg end) diff-ansi--ansi-color-timer)))
+        (timer-activate diff-ansi--ansi-color-timer)))))
 
 
 ;; ---------------------------------------------------------------------------
@@ -805,9 +812,9 @@ Store the result in TARGET-BUF when non-nil."
                    (emacs-eval-arg (diff-ansi--ansi-color-apply-on-region-with-bg-str black-color))
 
                    (point-prev (point))
-                   ;; Index (for unique names).
+                   ;; Index (for unique names.
                    (i 0))
-              (while (null (eobp))
+              (while (not (eobp))
                 (setq point-prev (point))
                 (goto-char (min (+ (point) diff-ansi-chunks-size) (point-max)))
                 (goto-char (pos-eol))
@@ -956,7 +963,7 @@ Return the buffer used to write data into on success."
 
     (let ((method diff-ansi-method))
 
-      ;; Fallback to immediate when the task won't be split.
+      ;; Fallback to immediate when the task wont be split.
       (when (<= (- end beg) diff-ansi-chunks-size)
         (setq method 'immediate))
 
@@ -972,7 +979,7 @@ Return the buffer used to write data into on success."
 
     ;; Update end.
     (unless target-buf
-      (setq end (- (buffer-size) end-trailing-chars)))
+      (setq end (- (point-max) end-trailing-chars)))
 
     ;; This is only an approximation, as the diff views may not be exactly compatible.
     (when preserve-point
@@ -994,8 +1001,7 @@ Return the buffer used to write data into on success."
 
     (cond
      (create-buffer
-      (with-current-buffer target-buf
-        (set-buffer-modified-p nil))
+      (set-buffer-modified-p nil)
       target-buf)
      (t
       (current-buffer)))))
@@ -1011,9 +1017,8 @@ Return the buffer used to write data into on success."
 ;;;###autoload
 (define-minor-mode diff-ansi-mode
   "Enable `diff-ansi' integration."
-  :global nil
-  :lighter nil
-
+  :global
+  nil
   (cond
    (diff-ansi-mode
     (diff-ansi--enable))
